# we got the power version II (run by drake)
# In this version we run and compare statistical power analysis using three different datasets:
# - Irish CER smart meter data
# - UK SAVE 'smart meter' data
# - NZ GREEN Grid 'smart meter' data

library(drake)
library(data.table)
library(here)
library(lubridate)

# parameters ----
cerPath <- "~/Data/CER_Smart_Metering_Project/data/processed/"
# data before intervention
cerOctElecF <- paste0(cerPath, "CER_October_2009_Residential.csv.gz") # <- half hourly kWh consumption
cerDecElecF <- paste0(cerPath, "CER_December_2009_Residential.csv.gz") # <- half hourly kWh consumption

savePath <- "~/Data/SAVE/processed/"
saveJanElecF <- paste0(savePath, "jan2017clamp15minuteData.csv.gz") # <- quarter hourly Wh consumption

nzggPath <- "~/Data/NZ_GREENGrid/safe/gridSpy/1min/data/"
nzggElecF <- paste0(nzggPath, "allHouseholds_totalW_long.csv.gz") # <- per minute power

# functions ----

labelEveningPeak <- function(dt){
  # r_localDateTime must exist
  dt[, peakLabel := "Other"]
  dt[lubridate::hour(r_localDateTime) >= 16 &
       lubridate::hour(r_localDateTime) < 20, # up to
     peakLabel := "Evening"]
  return(dt)
}

processDataGeneric <- function(dt){
  # data processing that is common to all
  dt <- dt[, r_dateTime := lubridate::as_datetime(r_dateTime)]
  dt <- labelEveningPeak(dt) # might break on NZ <-> UTC?
}

readData <- function(f){
  dt <- data.table::fread(f)
  return(dt)
}

getCERdata <- function(f){
  dt <- readData(f)
  # put data processing here
  setnames(dt, "r_datetime", "r_localDateTime")
  dt <- dt[, r_localDateTime := lubridate::as_datetime(r_localDateTime)]
  dt <- labelEveningPeak(dt)
  return(dt)
}

getSAVEdata <- function(f){
  dt <- readData(f)
  # put data processing here
  setnames(dt, "obsDateTime", "r_localDateTime")
  dt <- dt[, r_localDateTime := lubridate::as_datetime(r_localDateTime)]
  dt <- dt[, r_localDateTimeHalfHour := lubridate::floor_date(r_localDateTime, unit = "30 minutes")]
  dt <- dt[, .(kWh = sum(consumptionWh)/1000), # aggregate to half hours to match CER
           keyby = .(bmg_id, trialGroupNavetas, r_localDateTime = r_localDateTimeHalfHour)]
  dt <- labelEveningPeak(dt)
  return(dt)
}

getNZGGdata <- function(f){
  dt <- readData(f)
  # put data processing here
  setnames(dt, "time_utc", "r_dateTime")
  dt <- dt[, r_localDateTime := lubridate::as_datetime(r_dateTime, tz = "Pacific/Auckland")]
  dt <- dt[, r_localDateTimeHalfHour := lubridate::floor_date(r_localDateTime, unit = "30 minutes")]
  dt <- dt[, .(kWh = (mean(sumW)/1000)/2), # aggregate to half hours to match CER
           keyby = .(linkID, r_localDateTime = r_localDateTimeHalfHour)]
  dt <- labelEveningPeak(dt)
  return(dt)
}

# check local files ----
# file.exists("drake.Rmd")

# drake plan ----
thePlan <- drake::drake_plan(
  cerOctDT = getCERdata(cerOctElecF),
  cerDecDT = getCERdata(cerDecElecF),
  saveJanDT = getSAVEdata(saveJanElecF),
  nzggDT = getNZGGdata(nzggElecF),
  report = rmarkdown::render(
     input = paste0(here::here(), "/code/wgtpDrake.Rmd"),
     output_file = file_out(paste0(here::here(), "/output/wgtpDrake.html")),
     quiet = FALSE
   )
)


# test it ----
thePlan

config <- drake_config(thePlan)
vis_drake_graph(config)

# do it ----
make(thePlan)
